# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import sys
import time
import socket

# --- py3: port --- #
import six
# import httplib
# import urlparse
if six.PY2:
    import httplib
    import urlparse

if six.PY3:
    import http.client as httplib
    import urllib.parse as urlparse
    unicode = lambda x: str(x)

import tempfile
import multiprocessing

# --- py3: port --- #
# from StringIO import StringIO
from six.moves import StringIO


from django.conf import settings
from django.core.handlers.wsgi import WSGIHandler
from django.core.servers.basehttp import WSGIServer
from django.core.servers.basehttp import ServerHandler
from django.core.servers.basehttp import WSGIRequestHandler

try:
    from django.core.servers.basehttp import AdminMediaHandler
except ImportError:
    AdminMediaHandler = None
    
if 'django.contrib.staticfiles' in settings.INSTALLED_APPS:
    try:
        from django.contrib.staticfiles.handlers import StaticFilesHandler
    except ImportError:
        StaticFilesHandler = None
else:
    StaticFilesHandler = None
    
try:
    from django.utils.six.moves import socketserver
except ImportError:
    import SocketServer as socketserver

try:
    import SocketServer
    SocketServer.BaseServer.handle_error = lambda *args, **kw: None
except ImportError:
    pass

from lettuce.django import mail
from lettuce.registry import call_hook


def create_mail_queue():
    mail.queue = multiprocessing.Queue()
    return mail.queue


class LettuceServerException(socket.error):
    pass

keep_running = True


class MutedRequestHandler(WSGIRequestHandler):
    """ A RequestHandler that silences output, in order to don't
    mess with Lettuce's output"""

    dev_null = StringIO()

    def log_message(self, *args, **kw):
        pass  # do nothing

    def handle(self):
        """Handle a single HTTP request"""
        self.raw_requestline = self.rfile.readline()
        if not self.parse_request():  # An error code has been sent, just exit
            return

        handler = LettuceServerHandler(
            self.rfile,
            self.wfile,
            self.dev_null,
            self.get_environ(),
        )
        handler.request_handler = self      # backpointer for logging
        handler.run(self.server.get_app())


class LettuceServerHandler(ServerHandler):
    def handle_error(self, request, client_address):
        pass

    def finish_response(self):
        try:
            ServerHandler.finish_response(self)

        # avoiding broken pipes
        # http://code.djangoproject.com/ticket/4444
        except Exception:
            exc_type, exc_value = sys.exc_info()[:2]
            if not issubclass(exc_type, socket.error) or \
                   exc_value.args[0] is 32:
                raise


class ThreadedServer(multiprocessing.Process):
    """
    Runs django's builtin in background
    """
    daemon = True

    def __init__(self, address, port, mail_queue, threading=True, *args, **kw):
        multiprocessing.Process.__init__(self)
        self.address = address
        self.port = port
        self.lock = multiprocessing.Lock()
        self.mail_queue = mail_queue
        self.threading = threading
        
    def configure_mail_queue(self):
        mail.queue = self.mail_queue
        settings.EMAIL_BACKEND = \
            'lettuce.django.mail.backends.QueueEmailBackend'

    @staticmethod
    def get_real_address(address):
        if address == '0.0.0.0' or address == 'localhost':
            address = '127.0.0.1'

        return address

    def wait(self):
        address = ThreadedServer.get_real_address(self.address)

        while True:
            time.sleep(0.1)
            http = httplib.HTTPConnection(address, self.port, timeout=1)
            try:
                http.request("GET", "/")
            except socket.error:
                http.close()
                continue
            break

        self.lock.acquire()

    def should_serve_static_files(self):
        try:
            return (StaticFilesHandler is not None and
                    getattr(settings, 'STATIC_URL', False))
        except ImportError:
            return False

    def should_serve_admin_media(self):
        try:
            return (('django.contrib.admin' in settings.INSTALLED_APPS and
                     AdminMediaHandler) or
                    getattr(settings, 'LETTUCE_SERVE_ADMIN_MEDIA', False))
        except ImportError:
            return False

    def run(self):
        self.lock.acquire()
        pidfile = os.path.join(tempfile.gettempdir(), 'lettuce-django.pid')
        if os.path.exists(pidfile):
            pid = int(open(pidfile).read())
            try:
                os.kill(pid, 9)

            except OSError:
                pass

            finally:
                os.unlink(pidfile)

        open(pidfile, 'w').write(unicode(os.getpid()))

        self.configure_mail_queue()

        connector = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        if self.threading:
            httpd_cls = type(str('WSGIServer'), (socketserver.ThreadingMixIn, WSGIServer), {})
        else:
            httpd_cls = WSGIServer

        try:
            s = connector.connect((self.address, self.port))
            self.lock.release()
            os.kill(os.getpid(), 9)
        except socket.error:
            pass

        finally:
            self.lock.release()

        try:
            server_address = (self.address, self.port)
            httpd = httpd_cls(server_address, MutedRequestHandler)

        except socket.error:
            raise LettuceServerException(
                "the port %d already being used, could not start " \
                "django's builtin server on it" % self.port,
            )

        handler = WSGIHandler()
        if self.should_serve_admin_media():
            if not AdminMediaHandler:
                raise LettuceServerException(
                    "AdminMediaHandler is not available in this version of "
                    "Django. Please set LETTUCE_SERVE_ADMIN_MEDIA = False "
                    "in your Django settings.")
            admin_media_path = ''
            handler = AdminMediaHandler(handler, admin_media_path)

        if self.should_serve_static_files():
            handler = StaticFilesHandler(handler)

        httpd.set_app(handler)

        global keep_running
        while keep_running:
            call_hook('before', 'handle_request', httpd, self)
            httpd.handle_request()
            call_hook('after', 'handle_request', httpd, self)
            try:
                self.lock.release()
            except ValueError:
                pass


class BaseServer(object):
    """
    Base class for Lettuce's internal server
    """

    def __init__(self, address='0.0.0.0', port=None, threading=True):
        self.port = int(port or getattr(settings, 'LETTUCE_SERVER_PORT', 8000))
        self.address = unicode(address)
        self.threading = threading

    def start(self):
        """
        Starts the webserver and waits it to be available

        Chain this method up before your implementation
        """
        call_hook('before', 'runserver', self._server)

    def stop(self):
        """
        Stops the webserver

        Chain this method up after your implementation
        """

        call_hook('after', 'runserver', self._server)

    def url(self, url=''):
        """The url to access a server on"""
        raise NotImplemented()


class DefaultServer(BaseServer):
    """A silenced, lightweight and simple django's builtin server so
    that lettuce can be used with selenium, webdriver, windmill or any
    browser tool"""

    def __init__(self, *args, **kwargs):
        super(DefaultServer, self).__init__(*args, **kwargs)

        queue = create_mail_queue()
        self._server = ThreadedServer(self.address, self.port, queue, threading=self.threading)


    def start(self):
        super(DefaultServer, self).start()

        if self._server.should_serve_admin_media():
            msg = "Preparing to serve django's admin site static files"
            if getattr(settings, 'LETTUCE_SERVE_ADMIN_MEDIA', False):
                msg += ' (as per settings.LETTUCE_SERVE_ADMIN_MEDIA=True)'

            print("%s..." % msg)

        self._server.start()
        self._server.wait()

        addrport = self.address, self._server.port
        if not self._server.is_alive():
            raise LettuceServerException(
                'Lettuce could not run the builtin Django server at %s:%d"\n'
                'maybe you forgot a "runserver" instance running ?\n\n'
                'well if you really do not want lettuce to run the server '
                'for you, then just run:\n\n'
                'python manage.py --no-server' % addrport,
            )

        print("Django's builtin server is running at %s:%d" % addrport)

    def stop(self, fail=False):
        pid = self._server.pid
        if pid:
            os.kill(pid, 9)

        super(DefaultServer, self).stop()

        code = int(fail)
        return sys.exit(code)

    def url(self, url=""):
        base_url = "http://%s" % ThreadedServer.get_real_address(self.address)

        if self.port is not 80:
            base_url += ':%d' % self.port

        return urlparse.urljoin(base_url, url)


try:
    try:
        from django.contrib.staticfiles.testing import \
            StaticLiveServerTestCase as LiveServerTestCase
    except ImportError:
        from django.test.testcases import LiveServerTestCase

    class DjangoServer(BaseServer):
        """
        A server that uses Django's LiveServerTestCase to implement the Server class.
        """

        _server = None

        def start(self):
            super(DjangoServer, self).start()

            os.environ['DJANGO_LIVE_TEST_SERVER_ADDRESS'] = \
                    '{address}:{port}'.format(address=self.address,
                                              port=self.port)
            LiveServerTestCase.setUpClass()

            print("Django's builtin server is running at {address}:{port}".format(
                address=self.address,
                port=self.port))

        def stop(self, fail=False):
            LiveServerTestCase.tearDownClass()

            super(DjangoServer, self).stop()

            return 0

        def url(self, url=''):
            return urlparse.urljoin(
                'http://{address}:{port}/'.format(address=self.address,
                                                  port=self.port),
                url)

except ImportError:
    pass



# coding=utf-8
import shutil
from lettuce import world
from  optparse import  OptionParser
import os
import sys


world.fail_flag = False
world.skip_flag = False
world.re_flag = str()
world.case = list()
world.task_path = str()
world.task_name = str()
world.cache_path = str()


def wrapped():
    parser = OptionParser()
    parser.add_option("--lf", "--last-failed", dest="last_failed", action="store_true",
                      help="仅执行上次失败的测试案例")
    parser.add_option("--ff", "--failed-first", dest="failed_first", action="store_true",
                      help="优先执行上次失败的测试案例")
    parser.add_option("--cc", "--clear-cache", dest="clear_cache", action="store_true",
                      help="清除当前的执行缓存")
    # parser.add_option("--etn", "--error-task-name", dest="error_task_name",
    #                   help="指定需要运行的任务名称")
    parser.add_option("--tn", "--task-name", dest="task_name",
                      help="指定需要运行的任务名称")

    (options, args) = parser.parse_args()
    if options.clear_cache:
        if options.task_name:
            for i in os.listdir('./case_cache'):
                if options.task_name in i:
                    clear_cache(f'./case_cache/{i}')
        else:
            clear_cache('./case_cache')
    if options.task_name:
        world.task_name = options.task_name
    if options.last_failed:
        world.re_flag = 'lf'
    if options.failed_first:
        world.re_flag = 'ff'


def run_task(task_name: str) ->str:
    mkdir_cache()
    name = task_name.split('/')[-1]
    if world.task_name:
        task_name =  world.task_name
    else:
        task_name = name
    if world.re_flag:
        dir_path = 'case_cache'
        if world.re_flag == 'lf':
            # 清除上一次的执行缓存
            clear_cache(f'./case_cache/{task_name}_lf_cache')
            # 根据类型复制案例文件
            shutil.copyfile(f'./case_cache/{task_name}_cache/error_case.feature',
                            f'./case_cache/{task_name}/task.feature')
            world.cache_path = f'./case_cache/{task_name}_lf_cache'
        if world.re_flag == 'ff':
            # 清除上一次的执行缓存
            clear_cache(f'./case_cache/{task_name}_ff_cache')
            # 根据类型复制案例文件
            shutil.copyfile(f'./case_cache/{task_name}_cache/failed_first_case.feature',
                            f'./case_cache/{task_name}/task.feature')
            world.cache_path = f'./case_cache/{task_name}_ff_cache'
    else:
        # 清除上一次的执行缓存
        for i in [f'{task_name}', f'{task_name}_cache']:
            path = f'./case_cache/{i}'
            clear_cache(path)
        dir_path = 'features'
        world.cache_path = f'./case_cache/{task_name}_cache'
    if not os.path.exists(world.cache_path):
        os.mkdir(world.cache_path)
    return f'{dir_path}/{task_name}'


def mkdir_cache():
    if not os.path.exists('./case_cache'):
        os.mkdir('./case_cache')


def get_case_py_file(path):
    task_name = world.task_path.split('/')[-1]
    shutil.copytree(path, f'./case_cache/{task_name}', symlinks=False, ignore=None)
    for root, dirs, files in os.walk(f'./case_cache/{task_name}'):
        for i in files:
            if '.feature' in i:
                os.remove(f'./case_cache/{task_name}/{i}')
    


def clear_cache(path):
    if os.path.exists(path):
        shutil.rmtree(path, ignore_errors=False, onerror=None)


# def build_case():

    # _case = world.case[:]
    # temp_list = list()
    # for i in _case:
    #     a = i.replace("\033[1;37m", '')
    #     a = a.replace("\033[1;32m", '')
    #     a = a.replace("\033[1;30m", '')
    #     a = a.replace("\033[0m", '')
    #     a = a.replace("\033[A", '')
    #     a = a.replace("\033[0;36m", '')
    #     if a in temp_list:
    #         pass
    #     else:
    #         temp_list.append(a)
    #         if world.fail_flag or world.skip_flag:
    #             # 收集错误信息，保存在结果文件
    #             with open('res.txt', 'a+', encoding='utf8') as f:
    #                 f.write(f'{a}\n')
    #             # 收集错误案例详情，支持-lf指令执行上次失败的案例
    #             with open('./case_cache/error_case.feature', 'a+', encoding='utf8') as c:
    #                 for j in temp_list:
    #                     if "\033[0;31m" not in j and "\033[1;31m" not in j:
    #                         if "#" in j:
    #                             c.write(j.split('#')[0].rstrip() + '\n')
    #                         elif "|" in j:
    #                             c.write(j)
    #             world.fail_flag = False
    #             world.skip_flag = False
    #         else:
    #             # 创建临时文件，此处存放执行通过的案例，如果存在执行失败的案例此处临时文件会与错误案例文件合并，生成新案例文件,支持-ff指令优先执行上次执行失败的案例
    #             with open('./case_cache/temp_case.feature', 'a+', encoding='utf8') as t:
    #                 t.write(f'{a}\n')
    # world.case = []


def build_case():

    with open(f'{world.cache_path}/res.txt', 'a+', encoding='utf8') as f:
        with open(f'{world.cache_path}/error_case.feature', 'a+', encoding='utf8') as c:
            with open(f'{world.cache_path}/temp_case.feature', 'a+', encoding='utf8') as t:
                _case = world.case[:]
                temp_list = list()
                for i in _case:
                    a = i.replace("\033[1;37m", '')
                    a = a.replace("\033[1;32m", '')
                    a = a.replace("\033[1;30m", '')
                    a = a.replace("\033[0m", '')
                    a = a.replace("\033[A", '')
                    a = a.replace("\033[0;36m", '')
                    if not a.startswith("Feature"):
                        if a in temp_list:
                            pass
                        else:
                            temp_list.append(a)
                            if world.fail_flag is True or world.skip_flag is True:
                                # 收集错误信息，保存在结果文件
                                f.write(f'{a}\n')
                                # 收集错误案例详情，支持-lf指令执行上次失败的案例
                                if "\033[0;31m" not in a and "\033[1;31m" not in a:
                                    if "#" in a:
                                        c.write(a.split('#')[0].rstrip() + '\n')
                                    elif "|" in a:
                                        c.write(a)
                            else:
                                # 创建临时文件，此处存放执行通过的案例，如果存在执行失败的案例此处临时文件会与错误案例文件合并，
                                # 生成新案例文件,支持-ff指令优先执行上次执行失败的案例
                                if "#" in a:
                                    t.write(a.split('#')[0].rstrip() + '\n')
                                elif "|" in a:
                                    t.write(a)

    world.fail_flag = False
    world.skip_flag = False
    world.case = []


def build_failed_first_case():
    # 合并失败文件与通过文件
    if os.path.exists(f'{world.cache_path}/error_case.feature'):
        with open(f'{world.cache_path}/failed_first_case.feature', 'a+', encoding='utf8') as f:
            with open(f'{world.cache_path}/error_case.feature', 'r+', encoding='utf8') as c:
                with open(f'{world.cache_path}/temp_case.feature', 'r+', encoding='utf8') as t:
                    temp = c.readlines()
                    temp_b = t.readlines()
                    f.write("Feature: failed-first-case\n")
                    f.writelines(temp + temp_b)
        with open(f'{world.cache_path}/error_case.feature', 'w+', encoding='utf8') as u:
            u.write("Feature: error-case\n")
            u.writelines(temp)
    if not world.re_flag:
        get_case_py_file(world.task_path)
    os.remove(f'{world.cache_path}/temp_case.feature')
    
    
wrapped()
mkdir_cache()



import os
import sys
import commands

from tests.asserts import assert_not_equals
from lettuce.fs import FileSystem

current_directory = FileSystem.dirname(__file__)

OLD_PYTHONPATH = os.getenv('PYTHONPATH', ':'.join(sys.path))


def teardown():
    os.environ['PYTHONPATH'] = OLD_PYTHONPATH


def test_email():
    'lettuce should be able to receive emails sent from django server'
    os.environ['PYTHONPATH'] = current_directory
    os.environ['DJANGO_SETTINGS_MODULE'] = 'djangoapp'

    status, out = commands.getstatusoutput(
        "django-admin.py harvest email.feature --verbosity=2")

    assert_not_equals(status, 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import sys
import optparse

import lettuce


def main(args=sys.argv[1:]):
    base_path = os.path.join(os.path.dirname(os.curdir), 'features')
    parser = optparse.OptionParser(
        usage="%prog or type %prog -h (--help) for help",
        version=lettuce.version)

    parser.add_option("-v", "--verbosity",
                      dest="verbosity",
                      default=3,
                      help='The verbosity level')

    parser.add_option("--no-color",
                      action="store_true",
                      dest="no_color",
                      default=False,
                      help="Don't colorize the command output.")

    parser.add_option("-s", "--scenarios",
                      dest="scenarios",
                      default=None,
                      help='Comma separated list of scenarios to run')

    parser.add_option("-t", "--tag",
                      dest="tags",
                      default=None,
                      action='append',
                      help='Tells lettuce to run the specified tags only; '
                      'can be used multiple times to define more tags'
                      '(prefixing tags with "-" will exclude them and '
                      'prefixing with "~" will match approximate words)')

    parser.add_option("-r", "--random",
                      dest="random",
                      action="store_true",
                      default=False,
                      help="Run scenarios in a more random order to avoid interference")

    parser.add_option("--root-dir",
                      dest="root_dir",
                      default="/",
                      type="string",
                      help="Tells lettuce not to search for features/steps "
                      " above this directory.")

    parser.add_option("--with-xunit",
                      dest="enable_xunit",
                      action="store_true",
                      default=False,
                      help='Output JUnit XML test results to a file')

    parser.add_option("--xunit-file",
                      dest="xunit_file",
                      default=None,
                      type="string",
                      help='Write JUnit XML to this file. Defaults to '
                      'lettucetests.xml')

    parser.add_option("--with-subunit",
                      dest="enable_subunit",
                      action="store_true",
                      default=False,
                      help='Output Subunit test results to a file')

    parser.add_option("--subunit-file",
                      dest="subunit_filename",
                      default=None,
                      help='Write Subunit data to this file. Defaults to '
                      'subunit.bin')

    parser.add_option("--failfast",
                      dest="failfast",
                      default=False,
                      action="store_true",
                      help='Stop running in the first failure')

    parser.add_option("--pdb",
                      dest="auto_pdb",
                      default=False,
                      action="store_true",
                      help='Launches an interactive debugger upon error')

    options, args = parser.parse_args(args)
    if args:
        base_path = os.path.abspath(args[0])

    try:
        options.verbosity = int(options.verbosity)
    except ValueError:
        pass

    tags = None
    if options.tags:
        tags = [tag.strip('@') for tag in options.tags]

    runner = lettuce.Runner(
        base_path,
        scenarios=options.scenarios,
        verbosity=options.verbosity,
        random=options.random,
        enable_xunit=options.enable_xunit,
        xunit_filename=options.xunit_file,
        enable_subunit=options.enable_subunit,
        subunit_filename=options.subunit_filename,
        failfast=options.failfast,
        auto_pdb=options.auto_pdb,
        tags=tags,
        root_dir=options.root_dir,
    )

    result = runner.run()
    failed = result is None or result.steps != result.steps_passed
    raise SystemExit(int(failed))

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from lettuce.django.apps import harvest_lettuces

server = None
django_url = None


def get_server(*args, **kwargs):
    """
    Look up the server we are using and set it as the global
    """

    from django.conf import settings

    server_name = getattr(settings, 'LETTUCE_TEST_SERVER',
                          'lettuce.django.server.DefaultServer')
    module, klass = server_name.rsplit('.', 1)

    Server = getattr(__import__(module, fromlist=[klass]), klass)

    global server, django_url

    server = Server(*args, **kwargs)
    django_url = server.url

    return server


__all__ = ['harvest_lettuces', 'server', 'django_url']



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERsteps.pyCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# py3:mod
import six
import datetime
import sys
# py3:mod
from six.moves import StringIO

from lettuce.terrain import before, after

from subunit.v2 import StreamResultToBytes
from subunit.iso8601 import Utc


def open_file(filename):
    """
    open a subunit file

    this is not a context manager because it is used asynchronously by
    hooks

    out of the scope of enable() because we want to patch it in our tests
    """

    filename = filename or 'subunit.bin'

    return open(filename, 'wb')


def close_file(file_):
    """
    """

    file_.close()


def enable(filename=None):

    file_ = open_file(filename)

    streamresult = StreamResultToBytes(file_)
    streamresult.startTestRun()

    real_stdout = sys.stdout
    real_stderr = sys.stderr

    @before.each_scenario
    def before_scenario(scenario):

        # create redirects for stdout and stderr
        scenario.stdout = StringIO()
        scenario.stderr = StringIO()
        try:
            test_tags = scenario.tags
        except AttributeError:
            test_tags = ()

        streamresult.status(test_id=get_test_id(scenario),
                            test_status='inprogress',
                            test_tags=test_tags,
                            timestamp=now())


    @before.step_output
    def capture_output(step):

        # only consider steps for background
        if not step.scenario:
            return

        sys.stdout = step.scenario.stdout
        sys.stderr = step.scenario.stderr

    @after.step_output
    def uncapture_output(step):

        sys.stdout = real_stdout
        sys.stderr = real_stderr

    @after.each_scenario
    def after_scenario(scenario):

        streamresult.status(test_id=get_test_id(scenario),
                            file_name='stdout',
                            file_bytes=scenario.stdout.getvalue().encode('utf-8'),
                            mime_type='text/plain; charset=utf8',
                            eof=True)

        streamresult.status(test_id=get_test_id(scenario),
                            file_name='stderr',
                            file_bytes=scenario.stderr.getvalue().encode('utf-8'),
                            mime_type='text/plain; charset=utf8',
                            eof=True)

        if scenario.passed:
            streamresult.status(test_id=get_test_id(scenario),
                                test_status='success',
                                timestamp=now())
        else:
            streamresult.status(test_id=get_test_id(scenario),
                                test_status='fail',
                                timestamp=now())

    @after.each_step
    def after_step(step):

        # only consider steps for background
        if not step.scenario:
            return

        test_id = get_test_id(step.scenario)

        if step.passed:
            marker = u'✔'
        elif not step.defined_at:
            marker = u'?'
        elif step.failed:
            marker = u'❌'

            try:
                streamresult.status(test_id=test_id,
                                    file_name='traceback',
                                    file_bytes=step.why.traceback.encode('utf-8'),
                                    mime_type='text/plain; charset=utf8')
            except AttributeError:
                pass

        elif not step.ran:
            marker = u' '
        else:
            raise AssertionError("Internal error")

        steps = u'{marker} {sentence}\n'.format(
            marker=marker,
            sentence=step.sentence)
        streamresult.status(test_id=test_id,
                        file_name='steps',
                        file_bytes=steps.encode('utf-8'),
                        mime_type='text/plain; charset=utf8')

    @after.all
    def after_all(total):

        streamresult.stopTestRun()
        close_file(file_)


def get_test_id(scenario):
    try:
        return '{feature}: {scenario}'.format(
            feature=scenario.feature.name,
            scenario=scenario.name)
    except AttributeError:
        return '{feature}: Background'.format(
            feature=scenario.feature.name)


def now():
    """
    A timestamp suitable for subunit
    """

    return datetime.datetime.now(tz=Utc())



"""

A minimal Django app, just one file.

See: http://olifante.blogs.com/covil/2010/04/minimal-django.html

"""
import os
from django.conf.urls.defaults import patterns
from django.core.mail import send_mail

from django.http import HttpResponse
filepath, extension = os.path.splitext(__file__)
ROOT_URLCONF = os.path.basename(filepath)
INSTALLED_APPS = (
    "lettuce.django"
    )


def mail(request):
    send_mail('Subject here', 'Here is the message.', 'from@example.com',
             ['to@example.com'], fail_silently=False)
    return HttpResponse('Mail has been sent')

urlpatterns = patterns('', (r'^mail/$', mail))



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = version = '0.2.23'

release = 'kryptonite'

import os
import sys
import traceback
from importlib import reload
# from lettuce.plugins import non_verbose
# py3: mod
import six
try:
    # py3: mod
    if six.PY2:
        from imp import reload
    if six.PY3:
        from six.moves import reload_module as reload

except ImportError:
    # python 2.5 fallback
    pass

import random

from lettuce.core import Feature, TotalResult

from lettuce.terrain import after
from lettuce.terrain import before
from lettuce.terrain import world

from lettuce.decorators import step, steps
from lettuce.registry import call_hook
from lettuce.registry import STEP_REGISTRY
from lettuce.registry import CALLBACK_REGISTRY
from lettuce.exceptions import StepLoadingError, LettuceRunnerError
from lettuce.plugins import (
    xunit_output,
    subunit_output,
    autopdb,
    smtp_mail_queue,
    jsonreport_output,
)
from lettuce import fs
from lettuce import exceptions
from lettuce.cacheprovider import build_case, build_failed_first_case, run_task

try:
    from colorama import init as ms_windows_workaround
    ms_windows_workaround()
except ImportError:
    pass


__all__ = [
    'after',
    'before',
    'step',
    'steps',
    'world',
    'STEP_REGISTRY',
    'CALLBACK_REGISTRY',
    'call_hook',
    'build_case',
    'build_failed_first_case',
    'run_task',
]

try:
    # py3: mod
    if six.PY2:
        terrain = fs.FileSystem._import("terrain")

    if six.PY3:
        import importlib
        terrain = importlib.import_module("lettuce.terrain")
    reload(terrain)
except Exception as e:
    if not "No module named terrain" in str(e):
        string = 'Lettuce has tried to load the conventional environment ' \
            'module "terrain"\nbut it has errors, check its contents and ' \
            'try to run lettuce again.\n\nOriginal traceback below:\n\n'

        sys.stderr.write(string)
        sys.stderr.write(exceptions.traceback.format_exc(e))
        raise LettuceRunnerError(string)


class Runner(object):
    """ Main lettuce's test runner

    Takes a base path as parameter (string), so that it can look for
    features and step definitions on there.
    """
    def __init__(self, base_path, scenarios=None,
                 verbosity=0, no_color=False, random=False,
                 enable_xunit=False, xunit_filename=None,
                 enable_subunit=False, subunit_filename=None,
                 enable_jsonreport=False, jsonreport_filename=None,
                 tags=None, failfast=False, auto_pdb=False,
                 smtp_queue=None, root_dir=None):

        """ lettuce.Runner will try to find a terrain.py file and
        import it from within `base_path`
        """

        self.tags = tags
        self.single_feature = None

        if os.path.isfile(base_path) and os.path.exists(base_path):
            self.single_feature = base_path
            base_path = os.path.dirname(base_path)

        sys.path.insert(0, base_path)
        self.loader = fs.FeatureLoader(base_path, root_dir)
        self.verbosity = verbosity
        self.scenarios = scenarios and map(int, scenarios.split(",")) or None
        self.failfast = failfast
        if auto_pdb:
            autopdb.enable(self)

        sys.path.remove(base_path)

        if verbosity == 0:
            from lettuce.plugins import non_verbose as output
            reload(output)
            self.output = output
        elif verbosity == 1:
            from lettuce.plugins import dots as output
            reload(output)
            self.output = output
        elif verbosity == 2:
            from lettuce.plugins import scenario_names as output
            reload(output)
            self.output = output
        elif verbosity == 3:
            if no_color:
                from lettuce.plugins import shell_output as output
                reload(output)
                self.output = output
            else:
                from lettuce.plugins import colored_shell_output as output
                reload(output)
                self.output = output
        else:
            from lettuce.plugins import colored_shell_output as output
            reload(output)
            self.output = output


        self.random = random

        if enable_xunit:
            xunit_output.enable(filename=xunit_filename)
        if smtp_queue:
            smtp_mail_queue.enable()

        if enable_subunit:
            subunit_output.enable(filename=subunit_filename)

        if enable_jsonreport:
            jsonreport_output.enable(filename=jsonreport_filename)



    def run(self):
        """ Find and load step definitions, and them find and load
        features under `base_path` specified on constructor
        """
        results = []
        if self.single_feature:
            features_files = [self.single_feature]
        else:
            features_files = self.loader.find_feature_files()
            if self.random:
                random.shuffle(features_files)

        if not features_files:
            self.output.print_no_features_found(self.loader.base_dir)
            return

        # only load steps if we've located some features.
        # this prevents stupid bugs when loading django modules
        # that we don't even want to test.
        try:
            self.loader.find_and_load_step_definitions()
        except StepLoadingError as e:
            # py3: mod
            print("Error loading step definitions:\n", e)
            return

        call_hook('before', 'all')

        failed = False
        try:
            for filename in features_files:
                feature = Feature.from_file(filename)
                results.append(
                    feature.run(self.scenarios,
                                tags=self.tags,
                                random=self.random,
                                failfast=self.failfast))

        except exceptions.LettuceSyntaxError as e:
            sys.stderr.write(e.msg)
            failed = True
        # py3: mod
        except exceptions.NoDefinitionFound as e:
            sys.stderr.write(e.msg)
            failed = True
        except:
            if not self.failfast:
                e = sys.exc_info()[1]
                # py3: mod
                print("Died with %s" % str(e))
                traceback.print_exc()
            else:
                print
                print ("Lettuce aborted running any more tests "
                       "because was called with the `--failfast` option")

            failed = True

        finally:
            total = TotalResult(results)
            total.output_format()
            call_hook('after', 'all', total)

            if failed:
                raise LettuceRunnerError("Test failed.")

            return total
